import os
import pickle
from typing import TypeVar
from adolet_db.repo.postgres_repo import PostgresRepo
from adolet_db.repo.cache_postgres_repo import cache_postgres_repo

POSTGRESREPO = TypeVar('PostgresRepo')
CACHE_FILE = 'cache_postgres_repo.cache'


def repo(email: str, api_key: str) -> POSTGRESREPO:
    if os.path.exists(CACHE_FILE):
        # Load cache
        with open(CACHE_FILE, 'rb') as handle:
            postgres_repo = pickle.load(handle)
            return postgres_repo

    else:
        # Instantiate and authenticate Postgres Repo
        postgres_repo = PostgresRepo(email=email, api_key=api_key)
        postgres_repo.authenticate_db()

        # Save cache in .pkl file
        cache_postgres_repo(postgres_repo=postgres_repo)
        return postgres_repo



"""Utilities to lazily create and visit candidates found.

Creating and visiting a candidate is a *very* costly operation. It involves
fetching, extracting, potentially building modules from source, and verifying
distribution metadata. It is therefore crucial for performance to keep
everything here lazy all the way down, so we only touch candidates that we
absolutely need, and not "download the world" when we only need one version of
something.
"""

import functools

from pip._vendor.six.moves import collections_abc  # type: ignore

from pip._internal.utils.typing import MYPY_CHECK_RUNNING

if MYPY_CHECK_RUNNING:
    from typing import Callable, Iterator, Optional, Set, Tuple

    from pip._vendor.packaging.version import _BaseVersion

    from .base import Candidate

    IndexCandidateInfo = Tuple[_BaseVersion, Callable[[], Optional[Candidate]]]


def _iter_built(infos):
    # type: (Iterator[IndexCandidateInfo]) -> Iterator[Candidate]
    """Iterator for ``FoundCandidates``.

    This iterator is used the package is not already installed. Candidates
    from index come later in their normal ordering.
    """
    versions_found = set()  # type: Set[_BaseVersion]
    for version, func in infos:
        if version in versions_found:
            continue
        candidate = func()
        if candidate is None:
            continue
        yield candidate
        versions_found.add(version)


def _iter_built_with_prepended(installed, infos):
    # type: (Candidate, Iterator[IndexCandidateInfo]) -> Iterator[Candidate]
    """Iterator for ``FoundCandidates``.

    This iterator is used when the resolver prefers the already-installed
    candidate and NOT to upgrade. The installed candidate is therefore
    always yielded first, and candidates from index come later in their
    normal ordering, except skipped when the version is already installed.
    """
    yield installed
    versions_found = {installed.version}  # type: Set[_BaseVersion]
    for version, func in infos:
        if version in versions_found:
            continue
        candidate = func()
        if candidate is None:
            continue
        yield candidate
        versions_found.add(version)


def _iter_built_with_inserted(installed, infos):
    # type: (Candidate, Iterator[IndexCandidateInfo]) -> Iterator[Candidate]
    """Iterator for ``FoundCandidates``.

    This iterator is used when the resolver prefers to upgrade an
    already-installed package. Candidates from index are returned in their
    normal ordering, except replaced when the version is already installed.

    The implementation iterates through and yields other candidates, inserting
    the installed candidate exactly once before we start yielding older or
    equivalent candidates, or after all other candidates if they are all newer.
    """
    versions_found = set()  # type: Set[_BaseVersion]
    for version, func in infos:
        if version in versions_found:
            continue
        # If the installed candidate is better, yield it first.
        if installed.version >= version:
            yield installed
            versions_found.add(installed.version)
        candidate = func()
        if candidate is None:
            continue
        yield candidate
        versions_found.add(version)

    # If the installed candidate is older than all other candidates.
    if installed.version not in versions_found:
        yield installed


class FoundCandidates(collections_abc.Sequence):
    """A lazy sequence to provide candidates to the resolver.

    The intended usage is to return this from `find_matches()` so the resolver
    can iterate through the sequence multiple times, but only access the index
    page when remote packages are actually needed. This improve performances
    when suitable candidates are already installed on disk.
    """
    def __init__(
        self,
        get_infos,  # type: Callable[[], Iterator[IndexCandidateInfo]]
        installed,  # type: Optional[Candidate]
        prefers_installed,  # type: bool
    ):
        self._get_infos = get_infos
        self._installed = installed
        self._prefers_installed = prefers_installed

    def __getitem__(self, index):
        # type: (int) -> Candidate
        # Implemented to satisfy the ABC check. This is not needed by the
        # resolver, and should not be used by the provider either (for
        # performance reasons).
        raise NotImplementedError("don't do this")

    def __iter__(self):
        # type: () -> Iterator[Candidate]
        infos = self._get_infos()
        if not self._installed:
            return _iter_built(infos)
        if self._prefers_installed:
            return _iter_built_with_prepended(self._installed, infos)
        return _iter_built_with_inserted(self._installed, infos)

    def __len__(self):
        # type: () -> int
        # Implemented to satisfy the ABC check. This is not needed by the
        # resolver, and should not be used by the provider either (for
        # performance reasons).
        raise NotImplementedError("don't do this")

    @functools.lru_cache(maxsize=1)
    def __bool__(self):
        # type: () -> bool
        if self._prefers_installed and self._installed:
            return True
        return any(self)

    __nonzero__ = __bool__  # XXX: Python 2.



import datetime
import hashlib
import json
import logging
import os.path
import sys

from pip._vendor.packaging import version as packaging_version
from pip._vendor.six import ensure_binary

from pip._internal.index.collector import LinkCollector
from pip._internal.index.package_finder import PackageFinder
from pip._internal.models.selection_prefs import SelectionPreferences
from pip._internal.utils.filesystem import adjacent_tmp_file, check_path_owner, replace
from pip._internal.utils.misc import ensure_dir, get_distribution, get_installed_version
from pip._internal.utils.packaging import get_installer
from pip._internal.utils.typing import MYPY_CHECK_RUNNING

if MYPY_CHECK_RUNNING:
    import optparse
    from typing import Any, Dict

    from pip._internal.network.session import PipSession


SELFCHECK_DATE_FMT = "%Y-%m-%dT%H:%M:%SZ"


logger = logging.getLogger(__name__)


def _get_statefile_name(key):
    # type: (str) -> str
    key_bytes = ensure_binary(key)
    name = hashlib.sha224(key_bytes).hexdigest()
    return name


class SelfCheckState:
    def __init__(self, cache_dir):
        # type: (str) -> None
        self.state = {}  # type: Dict[str, Any]
        self.statefile_path = None

        # Try to load the existing state
        if cache_dir:
            self.statefile_path = os.path.join(
                cache_dir, "selfcheck", _get_statefile_name(self.key)
            )
            try:
                with open(self.statefile_path) as statefile:
                    self.state = json.load(statefile)
            except (OSError, ValueError, KeyError):
                # Explicitly suppressing exceptions, since we don't want to
                # error out if the cache file is invalid.
                pass

    @property
    def key(self):
        # type: () -> str
        return sys.prefix

    def save(self, pypi_version, current_time):
        # type: (str, datetime.datetime) -> None
        # If we do not have a path to cache in, don't bother saving.
        if not self.statefile_path:
            return

        # Check to make sure that we own the directory
        if not check_path_owner(os.path.dirname(self.statefile_path)):
            return

        # Now that we've ensured the directory is owned by this user, we'll go
        # ahead and make sure that all our directories are created.
        ensure_dir(os.path.dirname(self.statefile_path))

        state = {
            # Include the key so it's easy to tell which pip wrote the
            # file.
            "key": self.key,
            "last_check": current_time.strftime(SELFCHECK_DATE_FMT),
            "pypi_version": pypi_version,
        }

        text = json.dumps(state, sort_keys=True, separators=(",", ":"))

        with adjacent_tmp_file(self.statefile_path) as f:
            f.write(ensure_binary(text))

        try:
            # Since we have a prefix-specific state file, we can just
            # overwrite whatever is there, no need to check.
            replace(f.name, self.statefile_path)
        except OSError:
            # Best effort.
            pass


def was_installed_by_pip(pkg):
    # type: (str) -> bool
    """Checks whether pkg was installed by pip

    This is used not to display the upgrade message when pip is in fact
    installed by system package manager, such as dnf on Fedora.
    """
    dist = get_distribution(pkg)
    if not dist:
        return False
    return "pip" == get_installer(dist)


def pip_self_version_check(session, options):
    # type: (PipSession, optparse.Values) -> None
    """Check for an update for pip.

    Limit the frequency of checks to once per week. State is stored either in
    the active virtualenv or in the user's USER_CACHE_DIR keyed off the prefix
    of the pip script path.
    """
    installed_version = get_installed_version("pip")
    if not installed_version:
        return

    pip_version = packaging_version.parse(installed_version)
    pypi_version = None

    try:
        state = SelfCheckState(cache_dir=options.cache_dir)

        current_time = datetime.datetime.utcnow()
        # Determine if we need to refresh the state
        if "last_check" in state.state and "pypi_version" in state.state:
            last_check = datetime.datetime.strptime(
                state.state["last_check"],
                SELFCHECK_DATE_FMT
            )
            if (current_time - last_check).total_seconds() < 7 * 24 * 60 * 60:
                pypi_version = state.state["pypi_version"]

        # Refresh the version if we need to or just see if we need to warn
        if pypi_version is None:
            # Lets use PackageFinder to see what the latest pip version is
            link_collector = LinkCollector.create(
                session,
                options=options,
                suppress_no_index=True,
            )

            # Pass allow_yanked=False so we don't suggest upgrading to a
            # yanked version.
            selection_prefs = SelectionPreferences(
                allow_yanked=False,
                allow_all_prereleases=False,  # Explicitly set to False
            )

            finder = PackageFinder.create(
                link_collector=link_collector,
                selection_prefs=selection_prefs,
            )
            best_candidate = finder.find_best_candidate("pip").best_candidate
            if best_candidate is None:
                return
            pypi_version = str(best_candidate.version)

            # save that we've performed a check
            state.save(pypi_version, current_time)

        remote_version = packaging_version.parse(pypi_version)

        local_version_is_older = (
            pip_version < remote_version and
            pip_version.base_version != remote_version.base_version and
            was_installed_by_pip('pip')
        )

        # Determine if our pypi_version is older
        if not local_version_is_older:
            return

        # We cannot tell how the current pip is available in the current
        # command context, so be pragmatic here and suggest the command
        # that's always available. This does not accommodate spaces in
        # `sys.executable`.
        pip_cmd = f"{sys.executable} -m pip"
        logger.warning(
            "You are using pip version %s; however, version %s is "
            "available.\nYou should consider upgrading via the "
            "'%s install --upgrade pip' command.",
            pip_version, pypi_version, pip_cmd
        )
    except Exception:
        logger.debug(
            "There was an error checking the latest version of pip",
            exc_info=True,
        )



from typing import List, Any


def is_args_none(args: List[Any]) -> None:
    '''Check if any of the argumements is None'''
    if not all(args):
        raise Exception("ERROR - Arguments can't be None")



import unittest
import os
from typing import Dict, List, Any
from adolet_db import repo


class TestPostgresRepo(unittest.TestCase):
    '''PostgresRepo will be connected to "unittest" table in "adolet" ElephantSQL database'''
    def __init__(self, *args, **kwargs):
        super(TestPostgresRepo, self).__init__(*args, **kwargs)
        self.postgres_repo = repo(
            email='blah@gmail.com',
            api_key=os.environ.get('ADOLET_DB_API'),
        )

    def test_dt(self):
        '''Test .dt() of PostgresRepo'''
        tables = self.postgres_repo.dt()

        self.assertListEqual(
            tables,
            ['users_databases', 'unittest', 'users'],
        )

    def test_get_columns(self):
        '''Test that .get_columns() works and returns an array of column names of a table'''
        cols = self.postgres_repo.get_columns(table_name='unittest')

        self.assertListEqual(
            cols,
            ['id', 'name', 'age'],
        )

    def test_get_table_schema(self):
        '''Test .get_table_schema() of ClientPostgresRepo'''
        table_schema = self.postgres_repo.get_table_schema(
            table_name='unittest', )

        self.assertDictEqual(
            table_schema,
            {
                'id': 'integer',
                'name': 'text',
                'age': 'integer',
            },
        )

    def test_insert_and_delete(self):
        '''Test .insert() and .delete() of PostgresRepo'''
        dto = {'id': None, 'name': 'Tom', 'age': 20}

        inserted_dto = self.postgres_repo.insert('unittest', dto)
        deleted_dto = self.postgres_repo.delete('unittest', inserted_dto['id'])

        self.assertDictEqual(
            inserted_dto,
            deleted_dto,
        )

    def test_updgate(self):
        dto = {'id': None, 'name': 'Random Name', 'age': 100}
        inserted_dto = self.postgres_repo.insert('unittest', dto)

        updated_dto = self.postgres_repo.update(
            table_name='unittest',
            id=inserted_dto['id'],
            updates={
                'name': 'Random Name Updated',
                'age': 10,
            },
        )
        self.postgres_repo.delete('unittest', updated_dto['id'])

        # Update inserted_dto to match updated_dto
        inserted_dto['name'] = 'Random Name Updated'
        inserted_dto['age'] = 10

        self.assertDictEqual(
            inserted_dto,
            updated_dto,
        )

    def test_search_with_multiple_conditions(self):
        '''Test .search_with_multiple_conditions() of ClientPostgresRepo'''
        search_dto = {'name': 'Bob', 'age': 18}
        dto = self.postgres_repo.search_with_multiple_conditions(
            table_name='unittest',
            dto=search_dto,
        )

        self.assertDictEqual(
            dto,
            {
                'id': 1,
                'name': 'Bob',
                'age': 18,
            },
        )

    def test_run_query(self):
        '''Test .run_query() of ClientPostgresRepo'''
        dtos: List[Dict[str, Any]] = self.postgres_repo.run_query(
                sql_query="SELECT * FROM unittest WHERE id=1;", \
        )

        self.assertListEqual(
            dtos,
            [{
                'id': 1,
                'name': 'Bob',
                'age': 18
            }],
        )


if __name__ == '__main__':
    unittest.main()



import requests
import pickle
import os
from typing import Dict, Any, TypeVar, List
from adolet_db.common import is_args_none

CACHE_FILE = 'cache_postgres_repo.cache'


class PostgresRepo:
    def __init__(self, email: str, api_key: str):
        self.email = email
        self.api_key = api_key
        self.URL = 'http://api.postgres.adolet.com'
        # Save connection details upon instantiation
        self.conn_details: Dict[str, Any] = self.authenticate_db()

    def clear_cache(self) -> bool:
        if os.path.exists(CACHE_FILE):
            os.remove(CACHE_FILE)
            print("SUCCESS - Cache has been cleared")
            return True

        print("FAILURE - There is no cache to be clear")
        return False

    def authenticate_db(self) -> Dict[str, Any]:
        try:
            params = {'email': self.email, 'api_key': self.api_key}
            res = requests.get(
                f'{self.URL}/authenticate-db-api',
                params=params,
            )

            res = res.json()
            return res

        except:
            raise Exception(
                "There is an error with .authenticate_db() in postgres_repo.py - You may need to call .clear_cache()"
            )

    def dt(self) -> List[str]:
        is_args_none(args=list(locals().values()))
        try:
            params = {'email': self.email, 'api_key': self.api_key}

            # Send connection details in REST API if given
            if self.conn_details is not None:
                params['dbname'] = self.conn_details['dbname']
                params['user'] = self.conn_details['user']
                params['password'] = self.conn_details['password']
                params['host'] = self.conn_details['host']
                params['port'] = self.conn_details['port']

            res = requests.get(f'{self.URL}/dt-api', params=params)
            res: Dict[str, Any] = res.json()
            return res

        except:
            raise Exception(
                "There is an error with .dt() in postgres_repo.py - You may need to call .clear_cache()"
            )

    def get_columns(self, table_name: str) -> List[str]:
        is_args_none(args=list(locals().values()))
        try:
            params = {
                'email': self.email,
                'api_key': self.api_key,
                'table_name': table_name,
            }

            # Send connection details in REST API if given
            if self.conn_details is not None:
                params['dbname'] = self.conn_details['dbname']
                params['user'] = self.conn_details['user']
                params['password'] = self.conn_details['password']
                params['host'] = self.conn_details['host']
                params['port'] = self.conn_details['port']

            res = requests.get(f'{self.URL}/get-columns-api', params=params)
            res: Dict[str, Any] = res.json()
            return res

        except:
            raise Exception(
                "There is an error with .get_columns() in postgres_repo.py - You may need to call .clear_cache()"
            )

    def get_table_schema(self, table_name: str) -> Dict[str, str]:
        is_args_none(args=list(locals().values()))
        try:
            params = {
                'email': self.email,
                'api_key': self.api_key,
                'table_name': table_name,
            }

            # Send connection details in REST API if given
            if self.conn_details is not None:
                params['dbname'] = self.conn_details['dbname']
                params['user'] = self.conn_details['user']
                params['password'] = self.conn_details['password']
                params['host'] = self.conn_details['host']
                params['port'] = self.conn_details['port']

            res = requests.get(
                f'{self.URL}/get-table-schema-api',
                params=params,
            )
            res: Dict[str, Any] = res.json()
            return res

        except:
            raise Exception(
                "There is an error with .get_table_schema() in postgres_repo.py - You may need to call .clear_cache()"
            )

    def insert(self, table_name: str, dto: Dict[str, Any]) -> Dict[str, Any]:
        is_args_none(args=list(locals().values()))
        try:
            if 'id' in dto: dto.pop('id')
            params = {
                'email': self.email,
                'api_key': self.api_key,
                'table_name': table_name,
                'columns': list(dto.keys()),
                'values': list(dto.values()),
            }

            # Send connection details in REST API if given
            if self.conn_details is not None:
                params['dbname'] = self.conn_details['dbname']
                params['user'] = self.conn_details['user']
                params['password'] = self.conn_details['password']
                params['host'] = self.conn_details['host']
                params['port'] = self.conn_details['port']

            res = requests.get(f'{self.URL}/insert-api', params=params)
            res = res.json()
            return res

        except:
            raise Exception(
                "There is an error with .insert() in postgres_repo.py - You may need to call .clear_cache()"
            )

    def insert_many(
        self,
        table_name: str,
        dtos: List[Dict[str, Any]],
    ) -> List[Dict[str, Any]]:
        pass

    def delete(self, table_name: str, id: int) -> Dict[str, Any]:
        is_args_none(args=list(locals().values()))
        try:
            params = {
                'email': self.email,
                'api_key': self.api_key,
                'table_name': table_name,
                'id': int(id),
            }

            # Send connection details in REST API if given
            if self.conn_details is not None:
                params['dbname'] = self.conn_details['dbname']
                params['user'] = self.conn_details['user']
                params['password'] = self.conn_details['password']
                params['host'] = self.conn_details['host']
                params['port'] = self.conn_details['port']

            res = requests.get(f'{self.URL}/delete-api', params=params)
            res = res.json()
            return res

        except:
            raise Exception(
                "There is an error with .delete() in postgres_repo.py - You may need to call .clear_cache()"
            )

    def delete_many(self, table_name: str, ids: List[int]) -> Dict[str, Any]:
        pass

    def update(
        self,
        table_name: str,
        id: int,
        updates: Dict[str, Any],
    ) -> Dict[str, Any]:
        is_args_none(args=list(locals().values()))
        try:
            # Split updates into 2 arrays
            columns_of_updates = [column for column, _ in updates.items()]
            new_values_of_updates = [value for _, value in updates.items()]
            params = {
                'email': self.email,
                'api_key': self.api_key,
                'table_name': table_name,
                'id': id,
                'columns_of_updates': columns_of_updates,
                'new_values_of_updates': new_values_of_updates,
            }

            # Send connection details in REST API if given
            if self.conn_details is not None:
                params['dbname'] = self.conn_details['dbname']
                params['user'] = self.conn_details['user']
                params['password'] = self.conn_details['password']
                params['host'] = self.conn_details['host']
                params['port'] = self.conn_details['port']

            res = requests.get(f'{self.URL}/update-api', params=params)
            res: Dict[str, Any] = res.json()
            return res

        except:
            raise Exception(
                "There is an error with .update() in postgres_repo.py - You may need to call .clear_cache()"
            )

    def update_many(self):
        pass

    def find(self, table_name: str, id: int) -> Dict[str, Any]:
        pass

    def search_with_multiple_conditions(
        self,
        table_name: str,
        dto: Dict[str, Any],
    ) -> Dict[str, Any]:
        is_args_none(args=list(locals().values()))
        try:
            params = {
                'email': self.email,
                'api_key': self.api_key,
                'table_name': table_name,
                'columns': list(dto.keys()),
                'values': list(dto.values()),
            }

            # Send connection details in REST API if given
            if self.conn_details is not None:
                params['dbname'] = self.conn_details['dbname']
                params['user'] = self.conn_details['user']
                params['password'] = self.conn_details['password']
                params['host'] = self.conn_details['host']
                params['port'] = self.conn_details['port']

            res = requests.get(
                f'{self.URL}/search-with-multiple-conditions-api',
                params=params,
            )
            res: List[Dict[str, Any]] = res.json()
            print(res)
            return res

        except:
            raise Exception(
                "There is an error with .search_with_multiple_conditions() in postgres_repo.py - You may need to call .clear_cache()"
            )

    def run_query(self, sql_query: str) -> List[Dict[str, Any]]:
        is_args_none(args=list(locals().values()))
        try:
            params = {
                'email': self.email,
                'api_key': self.api_key,
                'sql_query': sql_query,
            }

            # Send connection details in REST API if given
            if self.conn_details is not None:
                params['dbname'] = self.conn_details['dbname']
                params['user'] = self.conn_details['user']
                params['password'] = self.conn_details['password']
                params['host'] = self.conn_details['host']
                params['port'] = self.conn_details['port']

            res = requests.get(f'{self.URL}/run-query-api', params=params)
            res: List[Dict[str, Any]] = res.json()
            return res

        except:
            raise Exception(
                "There is an error with .run_query() in postgres_repo.py - You may need to call .clear_cache()"
            )



from pip._vendor.resolvelib.providers import AbstractProvider

from pip._internal.utils.typing import MYPY_CHECK_RUNNING

from .base import Constraint

if MYPY_CHECK_RUNNING:
    from typing import Any, Dict, Iterable, Optional, Sequence, Set, Tuple, Union

    from .base import Candidate, Requirement
    from .factory import Factory

# Notes on the relationship between the provider, the factory, and the
# candidate and requirement classes.
#
# The provider is a direct implementation of the resolvelib class. Its role
# is to deliver the API that resolvelib expects.
#
# Rather than work with completely abstract "requirement" and "candidate"
# concepts as resolvelib does, pip has concrete classes implementing these two
# ideas. The API of Requirement and Candidate objects are defined in the base
# classes, but essentially map fairly directly to the equivalent provider
# methods. In particular, `find_matches` and `is_satisfied_by` are
# requirement methods, and `get_dependencies` is a candidate method.
#
# The factory is the interface to pip's internal mechanisms. It is stateless,
# and is created by the resolver and held as a property of the provider. It is
# responsible for creating Requirement and Candidate objects, and provides
# services to those objects (access to pip's finder and preparer).


class PipProvider(AbstractProvider):
    """Pip's provider implementation for resolvelib.

    :params constraints: A mapping of constraints specified by the user. Keys
        are canonicalized project names.
    :params ignore_dependencies: Whether the user specified ``--no-deps``.
    :params upgrade_strategy: The user-specified upgrade strategy.
    :params user_requested: A set of canonicalized package names that the user
        supplied for pip to install/upgrade.
    """

    def __init__(
        self,
        factory,  # type: Factory
        constraints,  # type: Dict[str, Constraint]
        ignore_dependencies,  # type: bool
        upgrade_strategy,  # type: str
        user_requested,  # type: Set[str]
    ):
        # type: (...) -> None
        self._factory = factory
        self._constraints = constraints
        self._ignore_dependencies = ignore_dependencies
        self._upgrade_strategy = upgrade_strategy
        self._user_requested = user_requested

    def identify(self, dependency):
        # type: (Union[Requirement, Candidate]) -> str
        return dependency.name

    def get_preference(
        self,
        resolution,  # type: Optional[Candidate]
        candidates,  # type: Sequence[Candidate]
        information  # type: Sequence[Tuple[Requirement, Candidate]]
    ):
        # type: (...) -> Any
        """Produce a sort key for given requirement based on preference.

        The lower the return value is, the more preferred this group of
        arguments is.

        Currently pip considers the followings in order:

        * Prefer if any of the known requirements points to an explicit URL.
        * If equal, prefer if any requirements contain ``===`` and ``==``.
        * If equal, prefer if requirements include version constraints, e.g.
          ``>=`` and ``<``.
        * If equal, prefer user-specified (non-transitive) requirements.
        * If equal, order alphabetically for consistency (helps debuggability).
        """

        def _get_restrictive_rating(requirements):
            # type: (Iterable[Requirement]) -> int
            """Rate how restrictive a set of requirements are.

            ``Requirement.get_candidate_lookup()`` returns a 2-tuple for
            lookup. The first element is ``Optional[Candidate]`` and the
            second ``Optional[InstallRequirement]``.

            * If the requirement is an explicit one, the explicitly-required
              candidate is returned as the first element.
            * If the requirement is based on a PEP 508 specifier, the backing
              ``InstallRequirement`` is returned as the second element.

            We use the first element to check whether there is an explicit
            requirement, and the second for equality operator.
            """
            lookups = (r.get_candidate_lookup() for r in requirements)
            cands, ireqs = zip(*lookups)
            if any(cand is not None for cand in cands):
                return 0
            spec_sets = (ireq.specifier for ireq in ireqs if ireq)
            operators = [
                specifier.operator
                for spec_set in spec_sets
                for specifier in spec_set
            ]
            if any(op in ("==", "===") for op in operators):
                return 1
            if operators:
                return 2
            # A "bare" requirement without any version requirements.
            return 3

        restrictive = _get_restrictive_rating(req for req, _ in information)
        transitive = all(parent is not None for _, parent in information)
        key = next(iter(candidates)).name if candidates else ""

        # HACK: Setuptools have a very long and solid backward compatibility
        # track record, and extremely few projects would request a narrow,
        # non-recent version range of it since that would break a lot things.
        # (Most projects specify it only to request for an installer feature,
        # which does not work, but that's another topic.) Intentionally
        # delaying Setuptools helps reduce branches the resolver has to check.
        # This serves as a temporary fix for issues like "apache-airlfow[all]"
        # while we work on "proper" branch pruning techniques.
        delay_this = (key == "setuptools")

        return (delay_this, restrictive, transitive, key)

    def find_matches(self, requirements):
        # type: (Sequence[Requirement]) -> Iterable[Candidate]
        if not requirements:
            return []
        name = requirements[0].project_name

        def _eligible_for_upgrade(name):
            # type: (str) -> bool
            """Are upgrades allowed for this project?

            This checks the upgrade strategy, and whether the project was one
            that the user specified in the command line, in order to decide
            whether we should upgrade if there's a newer version available.

            (Note that we don't need access to the `--upgrade` flag, because
            an upgrade strategy of "to-satisfy-only" means that `--upgrade`
            was not specified).
            """
            if self._upgrade_strategy == "eager":
                return True
            elif self._upgrade_strategy == "only-if-needed":
                return (name in self._user_requested)
            return False

        return self._factory.find_candidates(
            requirements,
            constraint=self._constraints.get(name, Constraint.empty()),
            prefers_installed=(not _eligible_for_upgrade(name)),
        )

    def is_satisfied_by(self, requirement, candidate):
        # type: (Requirement, Candidate) -> bool
        return requirement.is_satisfied_by(candidate)

    def get_dependencies(self, candidate):
        # type: (Candidate) -> Sequence[Requirement]
        with_requires = not self._ignore_dependencies
        return [
            r
            for r in candidate.iter_dependencies(with_requires)
            if r is not None
        ]



from pip._vendor.packaging.version import parse as parse_version

from pip._internal.utils.models import KeyBasedCompareMixin
from pip._internal.utils.typing import MYPY_CHECK_RUNNING

if MYPY_CHECK_RUNNING:
    from pip._vendor.packaging.version import _BaseVersion

    from pip._internal.models.link import Link


class InstallationCandidate(KeyBasedCompareMixin):
    """Represents a potential "candidate" for installation.
    """

    __slots__ = ["name", "version", "link"]

    def __init__(self, name, version, link):
        # type: (str, str, Link) -> None
        self.name = name
        self.version = parse_version(version)  # type: _BaseVersion
        self.link = link

        super().__init__(
            key=(self.name, self.version, self.link),
            defining_class=InstallationCandidate
        )

    def __repr__(self):
        # type: () -> str
        return "<InstallationCandidate({!r}, {!r}, {!r})>".format(
            self.name, self.version, self.link,
        )

    def __str__(self):
        # type: () -> str
        return '{!r} candidate (version {} at {})'.format(
            self.name, self.version, self.link,
        )



from setuptools import setup, find_packages

requirements = []
with open('requirements.txt') as f:
    requirements = f.readlines()

setup(
    name='adolet-db',
    packages=find_packages(),
    version='0.1.4',
    license='MIT',
    description='API for AdoletDB',
    author='Adolet',
    author_email='support@adolet.com',
    url='https://github.com/adolet/adolet-db/',
    download_url=
    'https://github.com/adolet/adolet-db/archive/refs/tags/v0.1.4.tar.gz',
    keywords=['Adolet', 'Adolet DB', 'Adolet ORM'],
    install_requires=requirements,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
